;
; Game concept for the Atari 2600 VCS
; Project name SCSIcide
;
; Copyright 2001
; By Joe Grand, jgrand@mindspring.com (http://www.mindspring.com/~jgrand/atari)
; Latest update 25.mar.2001
;
; dasm source.s -f3 -osource.bin to compile
;
; Tested with StellaX v1.1.3a
;
; DONE
;
; 1. Draw score, logo, initial playfield [initial playfield jg-0]
; 2. Enable movement up and down with joystick [joystick jg-1]
; 3. Display drive head on the right side of the screen
; 4. Draw playfield, with moving drive head [drivehead jg-2]
; 5. Added 6-digit score display routine with hexadecimal score
; 6. Added IncScore routine to add X (up to $FF) to the score,
;    handles rolling over. [hexscore jg-3]
; 7. Added data latency buffer (timer)
; 8. Added byte counter with color variation (matches score) [buffers jg-4]
;
; REMOVED FROM PREVIOUS VERSION
;
; o Draw new logo/title banner for bottom of the screen
;
; NEXT STEPS
;
; Add data blobs (see PCMSD source code)
;
; POTENTIAL GAME FEATURES
;
; o Need to read correct color data blocks in order to advance level
; o Bit size becomes smaller (fragmented) and faster as level increases
; o Latency buffer decreases when colored data block missed
; o Bad blocks are any data block read of the incorrect color
; o Title screen
; o Demo/attract mode
; o Paddle instead of joystick
; o Add sound
;

;
; Clear memory and registers
; Set up variables
; Loop:
;    Do the vertical blank
;    Do game calculations
;    Draw screen
;    Do more calculations during overscan
;    Wait for next vertical blank
; End Loop.
;

        processor 6502
        include vcs.h

        SEG.U vars
        org $80         ; Allocate memory, PIA has 128 bytes of RAM ($80-$FF). Stack from $FF

temp                    ds      1
saveStack               ds      1
dataIndex               ds      1       ; Counter for drawing
joystick0               ds      1       ; Player 1's joystick value
counter                 ds      1       ; Counter for drawing score

verPosP0                ds      1       ; Vertical position player 0 (drivehead)
;verPosP1                ds      10

horPosP0                ds      1       ; Horizontal position player 0 (drivehead)
;horPosP1                ds      10      ; Horizontal position player 1 (data)

zColorScore             ds      1
zColorDriveHead         ds      1
zColorData              ds      1
zColorLine              ds      1       ; Track seperator
zColorTrack             ds      1
zColorBackground        ds      1

currentDataColor        ds      1       ; Color of data to match

scorePtr0               ds      2       ; Pointer to score shapes
scorePtr1               ds      2
scorePtr2               ds      2
scorePtr3               ds      2
scorePtr4               ds      2
scorePtr5               ds      2

gauge                   ds      2       ; Data latency buffer, PF0 and PF1 (1st half of screen)
bytecount               ds      2       ; Number of data blocks read, PF1 and PF2 (2nd half of screen)


;carMotions              ds      10      ; Motion values for each car
;carMotionTimers         ds      10      ; Timer before car is moved
;zCarPatterns            ds      10      ; Current size and multiples of cars

;GameNumber             ds      1       ; Current game variation being played (0-7)
;CurrentLine            ds      1       ; Current line being drawn

; All variables from here to $FF are zero'd out every new game

score                   ds      3       ; Player score (6 hex digits)

        SEG code
        org $F800       ; Limit ourselves to 2K like the old days

Start

;
; The 2600 powers up in a completely random state, except for the PC which
; is set to the location at $FFC.  Therefore the first thing we must do is
; to set everything up inside the 6502.
;

        SEI             ; Disable interrupts, if there are any.
        CLD             ; Clear BCD math bit.
        LDX  #$FF
        TXS             ; Set stack to beginning.

        ; Loop to clear memory and TIA registers

        LDA  #0
B1      STA  0,X
        DEX
        BNE  B1

        ; The above routine does not clear location 0, which is VSYNC

        JSR  GameInit           ; Initial variable and register setup
MainLoop
        JSR  VerticalBlank      ; Execute the vertical blank.
        JSR  CheckSwitches      ; Check console switches.
        JSR  GameCalc           ; Do calculations during VBLANK
        JSR  DrawScreen         ; Draw the screen
        JSR  OverScan           ; Do more calculations during overscan
        JMP  MainLoop           ; Continue forever.


VerticalBlank  ;*********************** VERTICAL BLANK HANDLER

;
; Beginning of the frame - at the end of overscan.
;

        LDA  #0
        STA  VBLANK     ; Disable video (helpful for PAL systems?).

        LDA  #2         ; VBLANK was set at the beginning of overscan.
        STA  WSYNC
        STA  WSYNC
        STA  WSYNC

        STA  VSYNC      ; Begin vertical sync

        STA  WSYNC      ; First line of VSYNC
        STA  WSYNC      ; Second line of VSYNC

        ; Set the timer to go off just before the end of vertical blank space
        ; 37 scanlines * 76 cycles = 2812 cycles / 64 = approx. 44
        LDA  #44
        STA  TIM64T

;
; End the VSYNC period.
;
        LDA  #0
        STA  WSYNC ; Third line of VSYNC.
        STA  VSYNC ; (0)
        RTS


CheckSwitches ;*************************** CONSOLE SWITCH HANDLER

        ;LDA  #0         ; Clear collision latches
        ;STA  CXCLR

        LDA  SWCHA      ; Read joystick 0
        STA  joystick0  ; Store for later use

        LDA  SWCHB      ; Read the switches
        LSR             ; Shift the Reset switch
        ROR             ; Rotate Reset to Negative flag and Carry has select

        ; PAL/NTSC select check (use Difficulty switch?)

        RTS

;
; Minimal game calculations, just to get the ball rolling.
;
GameCalc ;******************************* GAME CALCULATION ROUTINES

        LDA  #$30
        STA  CTRLPF             ; No Reflect,No Score,Low Priority,8 pixel-wide ball

        LDX  #$FF               ; Temporary increment the score for demo purposes
        JSR  IncScore

        ;
        ; Horizontal position all objects (Courtesy of Manuel Polik)
        ;
        ;LDY  #11-1
        LDY  #1-1               ; Only set P0 for now
PosAnother
        LDX  horPosP0,Y
        JSR  PosPlayer
        DEY
        BPL  PosAnother

        STA  WSYNC              ; Finish current line
        STA  HMOVE

        ;
        ; Set new vertical position of drivehead based on joystick value
        ;
        LDA  joystick0          ; Load the joystick switches
        ROL
        ROL
        BMI  NotDown            ; Joystick pushed down?
        LDX  verPosP0
        CPX  BottomYPos
        BEQ  NoStick            ; If verPosP0 < BottomYPos, don't change
        DEC  verPosP0
        DEC  verPosP0
        JSR  DecGauge           ; Decrease gas gauge
        JSR  IncCount           ; Increase byte counter
        JMP  NoStick
NotDown
        ROL
        BMI  NoStick            ; Joystick pushed up?
        LDX  verPosP0
        CPX  TopYPos
        BEQ  NoStick            ; If verPosP0 > TopYPos, don't change
        INC  verPosP0
        INC  verPosP0
        JSR  IncGauge           ; Increase gas gauge
        LDA  #0                 ; Clear byte counter
        STA  bytecount
        STA  bytecount+1
        INC  currentDataColor
NoStick

        ;
        ; Set pointers to number data (based on current score)
        ; Score Score+1 Score+2
        ;
        LDX  #3-1
        LDY  #10
ScoreLoop
        LDA  score,X
        AND  #$0F               ; Mask it
        ASL                     ; Multiply by 8
        ASL
        ASL

        ADC  #<FontPage         ; Store the 2-byte pointer
        STA  scorePtr0,Y
        LDA  #0
        ADC  #>FontPage
        STA  scorePtr0+1,Y

        DEY
        DEY

        LDA  score,X
        AND  #$F0               ; Mask it
        LSR                     ; $00,$08,$10,$18,$20, ... ( / 8 = value)

        ADC  #<FontPage         ; Store the 2-byte pointer
        STA  scorePtr0,Y
        LDA  #0
        ADC  #>FontPage
        STA  scorePtr0+1,Y

        DEY
        DEY
        DEX
        BPL  ScoreLoop

        RTS


DrawScreen ;**************************** SCREEN DRAWING ROUTINES
;
; Initialize display variables.
;
        LDA  zColorBackground
        STA  COLUBK             ; Background will be black
        LDA  currentDataColor   ; Set the score color (P0 and P1)
        STA  COLUP0
        STA  COLUP1
        LDX  #3
        STX  NUSIZ0             ; Three copies of player, close together (for score)
        STX  NUSIZ1

WaitVBlank
        LDA  INTIM              ; Loop until timer is done - wait for the proper scanline
                                ; (i.e. somewhere in the last line of VBLANK)
        BNE  WaitVBlank         ; Whew! We're at the beginning of the frame
        LDA  #0
        STA  WSYNC              ; End the current scanline
        STA  HMOVE              ; Add horizontal motion to position sprites
        STA  VBLANK             ; End the VBLANK period with a zero, enable video.
        STA  WSYNC              ; Wait 2 scanlines before drawing score
        STA  WSYNC

        LDX  #22
Pause1
        DEX
        BNE  Pause1
        STA  RESP0

        STA  WSYNC
        LDX  #22
Pause2
        DEX
        BNE  Pause2
        NOP
        STA  RESP1
        STA  HMCLR
        LDA  #%10100000         ; P1 position 2 less than P0
        STA  HMP1
        LDA  #%11000000
        STA  HMP0
        STA  WSYNC
        STA  HMOVE
        STA  WSYNC


ScanLoop ; ================================== SCANNING LOOP
;
; There are 228/3 cycles per line, so we have a maximum of 73 cycles
; after STA WSYNC
;
; Display the scores - 6 digit, courtesy of Robin Harbron
;
        LDA  #8-1              ; Lines to display
        STA  counter

        LDX  #9
Pause3
        DEX
        BNE  Pause3
        NOP
        NOP
        NOP

        TSX
        STX  saveStack          ; Save the stack pointer (Andrew Davie)
DrawScore
        LDY  counter            ; 3
        LDA  (scorePtr0),Y      ; 5
        STA  GRP0               ; 3 = 11

        BIT  $00

        LDA  (scorePtr1),Y      ; 5
        STA  GRP1               ; 3
        LDA  (scorePtr5),Y      ; 5
        TAX                     ; 2
        TXS                     ; 2
        LDA  (scorePtr2),Y      ; 5
        STA  temp               ; 3
        LDA  (scorePtr3),Y      ; 5
        TAX                     ; 2
        LDA  (scorePtr4),Y      ; 5 = 37

        LDY  temp               ; 3
        STY  GRP0               ; 3
        STX  GRP1               ; 3
        STA  GRP0               ; 3
        TSX                     ; 2
        STX  GRP1               ; 3 = 17

        DEC  counter            ; 5
        BPL  DrawScore          ; 3 =  8

        LDX  saveStack
        TXS

        LDA  #0
        STA  GRP0
        STA  GRP1

        ;
        ; Skip down a few lines
        ; Do other calculations here if necessary, we have lots of time
        ;

        LDY  #6
DrawWait
        STA  WSYNC
        DEY
        BNE  DrawWait

        LDX  #0
        STX  NUSIZ0             ; One copy of player
        STX  NUSIZ1

        ; Start drawing the actual disk playfield (10 lanes, 15 lines each)
        LDX  horPosP0
        JSR  PosPlayer

        LDY  TopYPos            ; Set bounds on playfield
        LDA  zColorDriveHead    ; Setup colors
        STA  COLUP0
        LDA  zColorData
        STA  COLUP1

DrawTrack
        ;
        ; Is Y at the right position to draw drivehead?
        ;
        CPY  verPosP0
        BNE  NoYPos
        LDX  #16-1              ; Drivehead index
NoYPos
        CPX  #-1                ; If index < 0, graphics are finished drawing
        BEQ  NoHead

        ;
        ; Time to draw a line of the drivehead
        ;
        LDA  HeadFrame,X        ; Get the shape of the drivehead
        STA  GRP0               ; Draw it
        DEX
NoHead
        ;
        ; Calculate data positions and load graphics
        ;
;        STX  temp               ; Store X to temp since it is used while drawing drivehead
;        TYA
;        LSR                     ; Current track number = Y / 16
;        LSR
;        LSR
;        LSR
        ;STA  Score              ; Debug to show track number
        ;TAX

        ;LDA  #CarMotions,X     ; Fine horizontal position
        ;LDA  #0
        ;STA  HMOVE
        ;STA  HMP1

        ;STY  TempY1
        ;LDX  #1                ; P1
        ;LDA  #40
        ;JSR  SetMotionRegsX    ; Set proper X axis for data
        ;STA  HMOVE             ; Add horizontal motion to position sprites
        ;
        ;Time to draw a line of the data for the track we're currently in
        ;
        ;LDY  TempY1
        ;LDX  DataIndex
        ;LDA  DataFrame,X       ; Get the shape of the data
        ;STA  GRP1              ; Draw it
        ;DEC  DataIndex
        ;BPL  NoReset
        ;LDX  #16-1
        ;STX  DataIndex         ; Reset index

NoReset
        STA  WSYNC              ; Wait for sync
;        LDX  temp               ; Restore X
        TYA
        AND  #15                ; Modulo 16 ( % 2^n = AND (2^n)-1 )
        BNE  TrackColor
        LDA  zColorLine         ; Draw track seperator every 16th line
        JMP  SetColor
TrackColor
        LDA  zColorTrack
SetColor
        STA  COLUBK             ; Draw the track background, either seperator or not
        DEY
        BNE  DrawTrack

        STA  WSYNC              ; End the current scan line

        ;
        ; Skip down a few lines
        ; Do other calculations here if necessary, we have lots of time
        LDA  zColorLine
        STA  COLUBK
        STY  GRP0               ; Clear graphics
        STY  GRP1
        STA  WSYNC              ; Draw the bottom border of the disk

        LDA  zColorBackground   ; Black line
        STA  COLUBK
        LDY  #6
DrawWait2
        STA  WSYNC
        DEY
        BNE  DrawWait2

;
; Draw gas gauge/data latency and byte counter buffer
;


        LDX  #$07    ; number of lines alike
NxtLine:
        STA  WSYNC   ; wait for line to finish

        LDA  zColorScore        ; [0] +4 green
        STA  COLUPF             ; [4] +3
        LDA  zColorData         ; [7] +4 red
        STA  COLUBK             ; [11] +3 set background

        ; First half of screen
        LDA  gauge      ; [14] +3
        STA  PF0        ; [17] +3
        LDA  gauge+1    ; [20] +3
        STA  PF1        ; [23] +3
        LDA  #0         ; [26] +3
        STA  PF2        ; [29] +3

        NOP             ; [32] +2
        NOP             ; [34] +2
        NOP             ; [36] +2

        STA  COLUBK     ; [38] +3 end of gauge -> background color back to black
        STA  PF0        ; [41] +3

        ; Second half of screen - display number of bytes currently retrieved
        LDA  currentDataColor   ; [44] +4
        STA  COLUPF             ; [48] +3
        LDA  bytecount          ; [51] +4
        STA  PF1                ; [55] +3
        LDA  bytecount+1        ; [58] +4
        STA  PF2                ; [62] +3

        DEX             ; [65] +5
        BNE  NxtLine    ; [70] +3 (take branch)

;
; Prepare to draw the logo
; Waste 1 scan line
;
;        STA  HMOVE
;        NOP
;        NOP
;        NOP
;        NOP
;        STY  HMCLR      ; Clear horizontal motion registers
;        INY             ; X = 1
;        STY  NUSIZ0     ; 2 shapes, close together
;        STY  NUSIZ1
;        STA  RESP0      ; Set the X coordinate
;        STA  RESP1
;        LDA  #$10       ; Move P1 one pixel to the left
;        STA  HMP1
;        LDA  LogoColor
;        STA  COLUP0
;        STA  COLUP1

;
; Now draw the 8 lines
; Note: All the shapes end with a shape of zero. This way, I can assume
; the video display is pure black after the loop is finished.
;
;        LDX  #8-1               ; 8 scan lines
;DrawLogo
;        STA  WSYNC              ; 3 Wait for sync
;        STA  HMOVE              ; 3 Move the players (Only first time)
;        LDA  Logo1,X            ; 4 Draw the shape
;        STA  GRP0               ; 3
;        LDA  Logo2,X            ; 4 Draw in player #1
;        STA  GRP1               ; 3
;        NOP                     ; 2 Waste 4 cycles
;        NOP                     ; 2
;        LDA  Logo4,X            ; 4 Preload the final 2 bytes
;        TAY                     ; 2
;        LDA  Logo3,X            ; 4
;        STA  GRP0               ; 3 Store in shape (Cycle 34)
;        STY  GRP1               ; 3
;        STA  HMCLR              ; 3 Clear horiz. motion for future loops
;        LDA  LogoColor,X        ; 4 Load line color from table
;        STA  COLUP0             ; 3
;        STA  COLUP1             ; 3
;        DEX                     ; 2 All done?
;        BPL  DrawLogo           ; 3 Loop

;
; Clear all registers here to prevent any possible bleeding.
;
        LDA  #2
        STA  WSYNC      ; Finish this scanline.
        STA  VBLANK     ; Make TIA output invisible,
        ; Now we need to worry about it bleeding when we turn
        ; the TIA output back on.
        LDY  #0
        STY  PF0
        STY  PF1
        STY  PF2
        ;STY  GRP0
        ;STY  GRP1
        ;STY  ENAM0
        ;STY  ENAM1
        ;STY  ENABL
        RTS


OverScan   ;***************************** OVERSCAN CALCULATIONS
;
; For the Overscan routine, one might take the time to process such
; things as collisions.
;
        LDX  #30
KillLines
        STA  WSYNC
        DEX
        BNE  KillLines
        RTS


GameInit   ;***************************** INITIAL VARIABLE & REGISTER SETUP
;
; GameInit could conceivably be called when the Select key is pressed,
; or some other event.
;
        ;
        ;Load base colors
        ;
        LDX  #6-1
NextColor
        LDA  BaseColors,X       ; Get the colors
        STA  zColorScore,X      ; Save the adjusted colors
        ;CPX  #4
        ;BPL  NotHardwareYet
        ;STA  COLUP0,X          ; Set the hardware defaults (P0, P1, PF, BK)
NotHardwareYet
        DEX
        BPL  NextColor

        STA  currentDataColor
        ;
        ; Clear data pointers
        ;
        LDX  #11
        LDA  #0
Clear2
        STA  scorePtr0,X
        DEX
        BPL  Clear2

        STA  WSYNC
        LDY  #3
Pos0
        DEY
        BNE  Pos0
        STA  RESP0

        ; Score
        STA  score      ; A = 0
        STA  score+1
        STA  score+2

        ; Byte counter
        STA  bytecount
        STA  bytecount+1

        ; Gas gauge
        LDA  #$F0
        STA  gauge
        LDA  #$FF
        STA  gauge+1

        ;
        ; Setup sprite positions
        ;
        LDA  #88                ; Start with drivehead positioned in center
        STA  verPosP0

        LDA  #140
        STA  horPosP0

        ;LDA  #16-1             ; Width of track
        ;STA  DataIndex

;        LDX  #10-1              ; 10 lanes
;DataInit
;        LDA  #0
;        STA  horPosP1,X
        ;LDA  #1                 ; Allow motion in 2 frames
        ;STA  carMotionTimers,X
        ;LDA  CarColors,X       ; Copy the car colors
        ;STA  ZCarColors,X
        ;CLC
        ;LDA  GameNumber        ; Get the traffic pattern
        ;AND  #3
        ;TAY
        ;TXA
        ;ADC  Mul10Tbl1,Y       ; Mul 0-3 by 10
        ;TAY
        ;LDA  CarPatterns,Y     ; Save the pattern
        ;STA  ZCarPatterns,X
        ;LDA  #$60               ; Horizontal motion (+6 pixels)
        ;STA  carMotions,X
;        DEX
;        BPL  DataInit

        RTS

;
; Increase the score by X (up to $FF)
; Handles roll of score from 0000FF to 000100, etc.
;
IncScore
        CLC             ; Clear carry
        TXA             ; Score increment into accumulator
        LDX  #3-1
Loop
        ADC  score,X    ; Add LSB
        STA  score,X
        LDA  #0         ; MSB 2 bytes = 0
        DEX
        BPL  Loop
        RTS

;
; Increase/decrease the value of the data latency buffer by 1 pixel
;

IncGauge
        LDA  gauge
        CMP  #$F0
        BEQ  IncPF1
        ASL
        ORA  #$10
        STA  gauge
        JMP  IncDone
IncPF1
        LDA  gauge+1
        LSR
        ORA  #$80
        STA  gauge+1
IncDone
        RTS


DecGauge
        LDA  gauge+1
        CMP  #$0
        BEQ  DecPF0
        ASL
        STA  gauge+1
        JMP  DecDone
DecPF0
        LDA  gauge
        LSR
        AND  #$F0
        STA  gauge
DecDone
        RTS


;
; Increase the value of the byte counter by 2 pixels
;

IncCount
        LDA  bytecount
        CMP  #$FF
        BEQ  IncPF2
        LSR
        LSR
        ORA  #$C0
        STA  bytecount
        JMP  IncDone2
IncPF2
        LDA  bytecount+1
        ASL
        ASL
        ORA  #$03
        STA  bytecount+1
IncDone2
        RTS

;
; Set the horizontal position of X
;
PosPlayer
        STA  WSYNC              ; Begin scanline
        LDA  HorzTable,X        ; +4  7
        STA  HMP0,Y             ; +3  10
        AND  #$0F               ; +2  12
        TAX                     ; +2  14
P0      DEX                     ; +2  16
        BPL  P0
        STA  RESP0,Y
        RTS


;
; Logo bitmap, 32 x 8
;

;Logo1
;                .byte   %00000000 ; |        |
;                .byte   %11100111 ; |XXX  XXX|
;                .byte   %00010100 ; |   X X  |
;                .byte   %00010100 ; |   X X  |
;                .byte   %00010100 ; |   X X  |
;                .byte   %11110100 ; |XXXX X  |
;                .byte   %10000100 ; |X    X  |
;                .byte   %11110111 ; |XXXX XXX|
;
;Logo2
;                .byte   %00000000 ; |        |
;                .byte   %01110010 ; | XXX  X |
;                .byte   %00001010 ; |    X X |
;                .byte   %00001010 ; |    X X |
;                .byte   %00001010 ; |    X X |
;                .byte   %01111000 ; | XXXX   |
;                .byte   %01000010 ; | X    X |
;                .byte   %01111010 ; | XXXX X |
;
;Logo3
;                .byte   %00000000 ; |        |
;                .byte   %11101011 ; |XXX X XX|
;                .byte   %10001010 ; |X   X X |
;                .byte   %10001010 ; |X   X X |
;                .byte   %10001010 ; |X   X X |
;                .byte   %10000010 ; |X     X |
;                .byte   %10001010 ; |X   X X |
;                .byte   %11101011 ; |XXX X XX|
;
;Logo4
;                .byte   %00000000 ; |        |
;                .byte   %10011110 ; |X  XXXX |
;                .byte   %11010000 ; |XX X    |
;                .byte   %01010000 ; | X X    |
;                .byte   %01010000 ; | X X    |
;                .byte   %01011110 ; | X XXXX |
;                .byte   %11010000 ; |XX X    |
;                .byte   %10011110 ; |X  XXXX |

HeadFrame
                .byte   %00000000 ; |        |
                .byte   %00011110 ; |   XXXX |
                .byte   %00011110 ; |   XXXX |
                .byte   %00110011 ; |  XX  XX|
                .byte   %00110001 ; |  XX   X|
                .byte   %00111000 ; |  XXX   |
                .byte   %01001100 ; | X  XX  |
                .byte   %11111100 ; |XXXXXX  |
                .byte   %01001100 ; | X  XX  |
                .byte   %00111000 ; |  XXX   |
                .byte   %00110001 ; |  XX   X|
                .byte   %00110011 ; |  XX  XX|
                .byte   %00011110 ; |   XXXX |
                .byte   %00011110 ; |   XXXX |
                .byte   %00000000 ; |        |
                .byte   %00000000 ; |        |

;DataFrame
;                .byte   %00000000 ; |        |
;                .byte   %00000000 ; |        |
;                .byte   %01100110 ; | XX  XX |
;                .byte   %11111110 ; |XXXXXXX |
;                .byte   %11001111 ; |XX  XXXX|
;                .byte   %10110011 ; |X XX  XX|
;                .byte   %10110011 ; |X XX  XX|
;                .byte   %10110011 ; |X XX  XX|
;                .byte   %10110011 ; |X XX  XX|
;                .byte   %10110011 ; |X XX  XX|
;                .byte   %10110011 ; |X XX  XX|
;                .byte   %10110011 ; |X XX  XX|
;                .byte   %11001111 ; |XX  XXXX|
;                .byte   %11111110 ; |XXXXXXX |
;                .byte   %01100110 ; | XX  XX |
;                .byte   %00000000 ; |        |

;
; Size and shapes for each traffic pattern for the 8 games
;
;CarPatterns
;                HEX     50403020101020304050    ; Game #1,#5
;                HEX     40312213041514233241    ; Game #2,#6
;                HEX     46362016050016203646    ; Game #3,#7
;                HEX     05152515050515251505    ; Game #4,#8

;LogoColor               ;1   8    7    6    5    4    3    2
;                .byte   $7A, $00, $7A, $78, $76, $74, $76, $78

BaseColors
                .byte   $B8     ; Score/Time Remaining color (Green)
                .byte   $1E     ; Drive Head color (Yellow)
                .byte   $34     ; Data color (Red)
                .byte   $F4     ; Line color (Light Brown)
                .byte   $F2     ; Track color (Brown)
                .byte   $00     ; Background color (Black)

TopYPos         .byte   160     ; Top-most scanline of playfield
BottomYPos      .byte   14      ; Bottom-most scanline of playfield


;
; Must not cross page boundaries!!
;
        org $FD00

HorzTable
                .byte   $30,$20,$10,$00,$F0,$E0,$D0,$C0,$B0,$A0,$90
                .byte   $71,$61,$51,$41,$31,$21,$11,$01,$F1,$E1,$D1,$C1,$B1,$A1,$91
                .byte   $72,$62,$52,$42,$32,$22,$12,$02,$F2,$E2,$D2,$C2,$B2,$A2,$92
                .byte   $73,$63,$53,$43,$33,$23,$13,$03,$F3,$E3,$D3,$C3,$B3,$A3,$93
                .byte   $74,$64,$54,$44,$34,$24,$14,$04,$F4,$E4,$D4,$C4,$B4,$A4,$94
                .byte   $75,$65,$55,$45,$35,$25,$15,$05,$F5,$E5,$D5,$C5,$B5,$A5,$95
                .byte   $76,$66,$56,$46,$36,$26,$16,$06,$F6,$E6,$D6,$C6,$B6,$A6,$96
                .byte   $77,$67,$57,$47,$37,$27,$17,$07,$F7,$E7,$D7,$C7,$B7,$A7,$97
                .byte   $78,$68,$58,$48,$38,$28,$18,$08,$F8,$E8,$D8,$C8,$B8,$A8,$98
                .byte   $79,$69,$59,$49,$39,$29,$19,$09,$F9,$E9,$D9,$C9,$B9,$A9,$99
                .byte   $7A,$6A,$5A,$4A,$3A,$2A,$1A,$0A,$FA,$EA,$DA,$CA,$BA,$AA,$9A


; All shapes are upside down to allow decrementing by Y as both
; a counter and a shape index
;
        org $FE00 ; *********************** GRAPHICS DATA

;
; Numeric font
;
FontPage
Zero
                .byte   %00111100 ; |  XXXX  |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %00111100 ; |  XXXX  |

One
                .byte   %00111100 ; |  XXXX  |
                .byte   %00011000 ; |   XX   |
                .byte   %00011000 ; |   XX   |
                .byte   %00011000 ; |   XX   |
                .byte   %00011000 ; |   XX   |
                .byte   %00011000 ; |   XX   |
                .byte   %00111000 ; |  XXX   |
                .byte   %00011000 ; |   XX   |

Two
                .byte   %01111110 ; | XXXXXX |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %00111100 ; |  XXXX  |
                .byte   %00000110 ; |     XX |
                .byte   %00000110 ; |     XX |
                .byte   %01000110 ; | X   XX |
                .byte   %00111100 ; |  XXXX  |

Three
                .byte   %00111100 ; |  XXXX  |
                .byte   %01000110 ; | X   XX |
                .byte   %00000110 ; |     XX |
                .byte   %00001100 ; |    XX  |
                .byte   %00001100 ; |    XX  |
                .byte   %00000110 ; |     XX |
                .byte   %01000110 ; | X   XX |
                .byte   %00111100 ; |  XXXX  |

Four
                .byte   %00001100 ; |    XX  |
                .byte   %00001100 ; |    XX  |
                .byte   %00001100 ; |    XX  |
                .byte   %01111110 ; | XXXXXX |
                .byte   %01001100 ; | X  XX  |
                .byte   %00101100 ; |  X XX  |
                .byte   %00011100 ; |   XXX  |
                .byte   %00001100 ; |    XX  |

Five
                .byte   %01111100 ; | XXXXX  |
                .byte   %01000110 ; | X   XX |
                .byte   %00000110 ; |     XX |
                .byte   %00000110 ; |     XX |
                .byte   %01111100 ; | XXXXX  |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01111110 ; | XXXXXX |

Six
                .byte   %00111100 ; |  XXXX  |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01111100 ; | XXXXX  |
                .byte   %01100000 ; | XX     |
                .byte   %01100010 ; | XX   X |
                .byte   %00111100 ; |  XXXX  |

Seven
                .byte   %00011000 ; |   XX   |
                .byte   %00011000 ; |   XX   |
                .byte   %00011000 ; |   XX   |
                .byte   %00011000 ; |   XX   |
                .byte   %00001100 ; |    XX  |
                .byte   %00000110 ; |     XX |
                .byte   %01000010 ; | X    X |
                .byte   %01111110 ; | XXXXXX |

Eight
                .byte   %00111100 ; |  XXXX  |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %00111100 ; |  XXXX  |
                .byte   %00111100 ; |  XXXX  |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %00111100 ; |  XXXX  |

Nine
                .byte   %00111100 ; |  XXXX  |
                .byte   %01000110 ; | X   XX |
                .byte   %00000110 ; |     XX |
                .byte   %00111110 ; |  XXXXX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %00111100 ; |  XXXX  |

A
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01111110 ; | XXXXXX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %00111100 ; |  XXXX  |

B
                .byte   %01111100 ; | XXXXX  |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01111100 ; | XXXXX  |
                .byte   %01111100 ; | XXXXX  |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01111100 ; | XXXXX  |

C
                .byte   %00111100 ; |  XXXX  |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01100110 ; | XX  XX |
                .byte   %00111100 ; |  XXXX  |

D
                .byte   %01111100 ; | XXXXX  |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100010 ; | XX   X |
                .byte   %01100010 ; | XX   X |
                .byte   %01100010 ; | XX   X |
                .byte   %01100010 ; | XX   X |
                .byte   %01100110 ; | XX  XX |
                .byte   %01111100 ; | XXXXX  |

E
                .byte   %01111110 ; | XXXXXX |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01111110 ; | XXXXXX |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01111110 ; | XXXXXX |

F
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01111110 ; | XXXXXX |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01111110 ; | XXXXXX |


        .byte "J. Grand"

;
; Avoid $FFF8 to allow use with Supercharger
;

;
; Set up the 6502 interrupt vector table
;
        org $FFFC

        .word Start     ; Reset
        .word Start     ; IRQ

        END